/************************************************************
 * Infinite Universe Engine:				                *
 *                                                          *
 * Interactive simulation of a solar system and             *
 * real-time rendering of planetary bodies with OpenGL      *
 * copyright  2002 Jochen Winzen                           *
 * winzen@online.de                                         *
 * -------------------------------------------------------- *
 * File:	JWInfiniteUniverseEngine.cpp                    *
 *                                                          *
 * Content: Implementation of GUI adapter class             *
 *          Here we combine the GUI widgets with functions  *
 *          of our engine classes                           *
 *                                                          *
 * Author:  Jochen Winzen                                   *
 * Version: 2002/12/08                                      *
 ************************************************************
 */
#include <algorithm>
#include <qstring>
#include <qcombobox>
#include "JWInfiniteUniverseEngine.h"
#include "../JWGameEngine.h"

void JWInfiniteUniverseEngine::initPlanetchoice(void)
{
	// Sort the vector of planets alphabetically
	std::sort(jwengine::jwgame->sol.components.begin(), jwengine::jwgame->sol.components.end(), C3DBase::alphaSort);

	// Iterator is used to loop through the vector sol.planets.
	std::vector<C3DBase *>::iterator theIterator;
	QString planetName;
	// Lookup contents of the vector sol.planets.
	for (theIterator = jwengine::jwgame->sol.components.begin(); theIterator != jwengine::jwgame->sol.components.end(); theIterator++)
	{
		// get name field of planet
		planetName = (*theIterator)->getName();
		// fill it into the combobox list
		planetchoice->insertItem(planetName);
	}

	// now activate the autopilot button
    connect((QObject *)fly_button, SIGNAL(clicked()), this, SLOT(startAutopilot()));
    connect((QObject *)planetchoice, SIGNAL(highlighted(const QString &)), this, SLOT(initDEMchoice(const QString &)));
}

void JWInfiniteUniverseEngine::startAutopilot(void)
{
	qWarning("Now warping to %s", (const char *)(planetchoice->currentText()));
	C3DBase *pdest = jwengine::jwgame->sol.getObjectByName((const char *)(planetchoice->currentText()));

	// check if current text of DEMchoice is sensible, then use this location!
	if (!(DEMchoice->currentText()).isEmpty())
		pdest = pdest->getObjectByName((const char *)(DEMchoice->currentText()));

  jwengine::jwgame->QualityScale = 0; // important because of extreme relocation

	C3DBase::warpToPosition(jwengine::jwgame->playerModel, pdest);

	jwengine::jwgame->setFocus();
}

void JWInfiniteUniverseEngine::initDEMchoice(const QString &pname)
{
	DEMchoice->clear();
	C3DBase *obj = jwengine::jwgame->sol.getObjectByName((const char*)pname);
	if (!obj->components.empty())
	{
		// Sort the vector of DEMs alphabetically
		std::sort(obj->components.begin(), obj->components.end(), C3DBase::alphaSort);
		// Iterator is used to loop through the vector sol.planets.
		std::vector<C3DBase *>::iterator theIterator;
		QString DEMName;
		// Lookup contents of the vector sol.planets.
		for (theIterator = obj->components.begin(); theIterator != obj->components.end(); theIterator++)
		{
			// get name field of planet
			DEMName = (*theIterator)->getName();
			// fill it into the combobox list
			DEMchoice->insertItem(DEMName);
		}
	}
}


