/*
	The Universe Development Kit
	Copyright (C) 2000  Sean O'Neil
	s_p_oneil@hotmail.com

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Matrix.h"

// as you can see this returns a simple ROTATION matrix (no translation!)
void CDoubleMatrix::operator=(const CDoubleQuaternion &q)
{
	// 9 muls, 15 adds
	double x2 = q.x + q.x, y2 = q.y + q.y, z2 = q.z + q.z;
	double xx = q.x * x2, xy = q.x * y2, xz = q.x * z2;
	double yy = q.y * y2, yz = q.y * z2, zz = q.z * z2;
	double wx = q.w * x2, wy = q.w * y2, wz = q.w * z2;

	f2[0][3] = f2[1][3] = f2[2][3] = f2[3][0] = f2[3][1] = f2[3][2] = 0; f2[3][3] = 1;
	f2[0][0] = 1-(yy+zz);	f2[0][1] = xy+wz;		f2[0][2] = xz-wy;
	f2[1][0] = xy-wz;		f2[1][1] = 1-(xx+zz);	f2[1][2] = yz+wx;
	f2[2][0] = xz+wy;		f2[2][1] = yz-wx;		f2[2][2] = 1-(xx+yy);
	/*
	this part MUST be transposed to be correct!
	f2[0][0] = 1-(yy+zz);	f2[1][0] = xy+wz;		f2[2][0] = xz-wy;
	f2[0][1] = xy-wz;		f2[1][1] = 1-(xx+zz);	f2[2][1] = yz+wx;
	f2[0][2] = xz+wy;		f2[1][2] = yz-wx;		f2[2][2] = 1-(xx+yy);
	*/
}

CDoubleMatrix CDoubleMatrix::operator*(const CDoubleMatrix &m) const
{
		// 36 muls, 27 adds
		// | f2[0][0] f2[1][0] f2[2][0] f2[3][0] |   | m.f2[0][0] m.f2[1][0] m.f2[2][0] m.f2[3][0] |   | f2[0][0]*m.f2[0][0]+f2[1][0]*m.f2[0][1]+f2[2][0]*m.f2[0][2] f2[0][0]*m.f2[1][0]+f2[1][0]*m.f2[1][1]+f2[2][0]*m.f2[1][2] f2[0][0]*m.f2[2][0]+f2[1][0]*m.f2[2][1]+f2[2][0]*m.f2[2][2] f2[0][0]*m.f2[3][0]+f2[1][0]*m.f2[3][1]+f2[2][0]*m.f2[3][2]+f2[3][0] |
		// | f2[0][1] f2[1][1] f2[2][1] f2[3][1] |   | m.f2[0][1] m.f2[1][1] m.f2[2][1] m.f2[3][1] |   | f2[0][1]*m.f2[0][0]+f2[1][1]*m.f2[0][1]+f2[2][1]*m.f2[0][2] f2[0][1]*m.f2[1][0]+f2[1][1]*m.f2[1][1]+f2[2][1]*m.f2[1][2] f2[0][1]*m.f2[2][0]+f2[1][1]*m.f2[2][1]+f2[2][1]*m.f2[2][2] f2[0][1]*m.f2[3][0]+f2[1][1]*m.f2[3][1]+f2[2][1]*m.f2[3][2]+f2[3][1] |
		// | f2[0][2] f2[1][2] f2[2][2] f2[3][2] | * | m.f2[0][2] m.f2[1][2] m.f2[2][2] m.f2[3][2] | = | f2[0][2]*m.f2[0][0]+f2[1][2]*m.f2[0][1]+f2[2][2]*m.f2[0][2] f2[0][2]*m.f2[1][0]+f2[1][2]*m.f2[1][1]+f2[2][2]*m.f2[1][2] f2[0][2]*m.f2[2][0]+f2[1][2]*m.f2[2][1]+f2[2][2]*m.f2[2][2] f2[0][2]*m.f2[3][0]+f2[1][2]*m.f2[3][1]+f2[2][2]*m.f2[3][2]+f2[3][2] |
		// | 0   0   0   1   |   | 0     0     0     1     |   | 0                             0                             0                             1                                 |
		CDoubleMatrix mRet;
		mRet.f2[0][0] = f2[0][0]*m.f2[0][0]+f2[1][0]*m.f2[0][1]+f2[2][0]*m.f2[0][2];
		mRet.f2[1][0] = f2[0][0]*m.f2[1][0]+f2[1][0]*m.f2[1][1]+f2[2][0]*m.f2[1][2];
		mRet.f2[2][0] = f2[0][0]*m.f2[2][0]+f2[1][0]*m.f2[2][1]+f2[2][0]*m.f2[2][2];
		mRet.f2[3][0] = f2[0][0]*m.f2[3][0]+f2[1][0]*m.f2[3][1]+f2[2][0]*m.f2[3][2]+f2[3][0];
		mRet.f2[0][1] = f2[0][1]*m.f2[0][0]+f2[1][1]*m.f2[0][1]+f2[2][1]*m.f2[0][2];
		mRet.f2[1][1] = f2[0][1]*m.f2[1][0]+f2[1][1]*m.f2[1][1]+f2[2][1]*m.f2[1][2];
		mRet.f2[2][1] = f2[0][1]*m.f2[2][0]+f2[1][1]*m.f2[2][1]+f2[2][1]*m.f2[2][2];
		mRet.f2[3][1] = f2[0][1]*m.f2[3][0]+f2[1][1]*m.f2[3][1]+f2[2][1]*m.f2[3][2]+f2[3][1];
		mRet.f2[0][2] = f2[0][2]*m.f2[0][0]+f2[1][2]*m.f2[0][1]+f2[2][2]*m.f2[0][2];
		mRet.f2[1][2] = f2[0][2]*m.f2[1][0]+f2[1][2]*m.f2[1][1]+f2[2][2]*m.f2[1][2];
		mRet.f2[2][2] = f2[0][2]*m.f2[2][0]+f2[1][2]*m.f2[2][1]+f2[2][2]*m.f2[2][2];
		mRet.f2[3][2] = f2[0][2]*m.f2[3][0]+f2[1][2]*m.f2[3][1]+f2[2][2]*m.f2[3][2]+f2[3][2];
		mRet.f2[0][3] = mRet.f2[1][3] = mRet.f2[2][3] = 0;
		mRet.f2[3][3] = 1;
		return mRet;
}

void CDoubleQuaternion::operator=(const CDoubleMatrix &m)
{
	// Check the sum of the diagonal
	double tr = m(0, 0) + m(1, 1) + m(2, 2);
	if(tr > 0.0f)
	{
		// The sum is positive
		// 4 muls, 1 div, 6 adds, 1 trig function call
		double s = sqrt(tr + 1.0);
		w = s * 0.5;
		s = 0.5 / s;
		x = (m(1, 2) - m(2, 1)) * s;
		y = (m(2, 0) - m(0, 2)) * s;
		z = (m(0, 1) - m(1, 0)) * s;
	}
	else
	{
		// The sum is negative
		// 4 muls, 1 div, 8 adds, 1 trig function call
		const int nIndex[3] = {1, 2, 0};
		int i, j, k;
		i = 0;
		if(m(1, 1) > m(i, i))
			i = 1;
		if(m(2, 2) > m(i, i))
			i = 2;
		j = nIndex[i];
		k = nIndex[j];

		double s = sqrt((m(i, i) - (m(j, j) + m(k, k))) + 1.0);
		(*this)[i] = s * 0.5;
		if(s != 0.0)
			s = 0.5 / s;
		(*this)[j] = (m(i, j) + m(j, i)) * s;
		(*this)[k] = (m(i, k) + m(k, i)) * s;
		(*this)[3] = (m(j, k) - m(k, j)) * s;
	}
}

CDoubleQuaternion CDoubleQuaternion::operator*(const CDoubleQuaternion &q) const
{
	// 12 muls, 30 adds
	double E = (x + z)*(q.x + q.y);
	double F = (z - x)*(q.x - q.y);
	double G = (w + y)*(q.w - q.z);
	double H = (w - y)*(q.w + q.z);
	double A = F - E;
	double B = F + E;
	return CDoubleQuaternion(
		(w + x)*(q.w + q.x) + (A - G - H) * 0.5f,
		(w - x)*(q.y + q.z) + (B + G - H) * 0.5f,
		(y + z)*(q.w - q.x) + (B - G + H) * 0.5f,
		(z - y)*(q.y - q.z) + (A + G + H) * 0.5f);
}

// Spherical linear interpolation between two quaternions
CDoubleQuaternion Slerp(const CDoubleQuaternion &q1, const CDoubleQuaternion &q2, const double t)
{
	// Calculate the cosine of the angle between the two
	double fScale0, fScale1;
	double dCos = q1.x * q2.x + q1.y * q2.y + q1.z * q2.z + q1.w * q2.w;

	// If the angle is significant, use the spherical interpolation
	if((1.0 - ABS(dCos)) > DELTA)
	{
		double dTemp = acos(ABS(dCos));
		double dSin = sin(dTemp);
		fScale0 = (double)(sin((1.0 - t) * dTemp) / dSin);
		fScale1 = (double)(sin(t * dTemp) / dSin);
	}
	// Else use the cheaper linear interpolation
	else
	{
		fScale0 = 1.0f - t;
		fScale1 = t;
	}
	if(dCos < 0.0)
		fScale1 = -fScale1;

	// Return the interpolated result
	return (q1 * fScale0) + (q2 * fScale1);
}



