/************************************************************
 * Infinite Universe Engine:				                *
 *                                                          *
 * Interactive simulation of a solar system and             *
 * real-time rendering of planetary bodies with OpenGL      *
 * copyright  2002 Jochen Winzen                           *
 * winzen@online.de                                         *
 * -------------------------------------------------------- *
 * File:	Starfield.h                                     *
 *                                                          *
 * Content: Header file of a starfield background class     *
 *                                                          *
 * Author:  Jochen Winzen                                   *
 * Version: 2002/12/08                                      *
 ************************************************************
 */

#ifndef JW_STARFIELD
#define JW_STARFIELD

#include "Matrix.h"
#include "JWGameEngine.h"

class Starfield : public C3DObject{

#define MAXSTARS 10000

public:
	float stars[3*MAXSTARS];

	CRandom rn;

	Starfield() {
		int i;
		CDoubleVector rndVec;

		rn.Init(123);

		for (i=0; i<MAXSTARS; i++)
		{
			rndVec = CDoubleVector(rn.RandomD(-1000,1000),rn.RandomD(-1000,1000),rn.RandomD(-1000,1000));
			rndVec.Normalize();
			rndVec *= 100000;	// to avoid collision with near plane
			stars[3*i +0] = (float)rndVec.x;
			stars[3*i +1] = (float)rndVec.y;
			stars[3*i +2] = (float)rndVec.z;
		}
	};

	void draw(C3DCamera* pCamera)
	{
		// first check if we are in a dense atmosphere so that stars are invisible
		if (jwengine::jwgame->starAlpha <= 0)
			return;

		glPushMatrix();
		// VERY vERY verY important!!!!!! do matrix generation by yourself!!!!
		glLoadMatrixd(pCamera->GetCameraMatrix(0));

		// transform finished, now draw stars
		//glDisable(GL_ALPHA_TEST);
 		//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		//glEnable(GL_BLEND);
		glColor4f(1.0f,1.0f,1.0f,jwengine::jwgame->starAlpha);
		glPointSize(1.0);
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_LIGHTING);
		glEnableClientState (GL_VERTEX_ARRAY);
		glVertexPointer (3, GL_FLOAT, 0, stars); // nice & fast!
		glDrawArrays(GL_POINTS, 0, MAXSTARS);
		glDisableClientState (GL_VERTEX_ARRAY);
		glEnable(GL_LIGHTING);			// Turn on lighting
		glEnable(GL_DEPTH_TEST);
		//glDisable(GL_BLEND);
		glPopMatrix();		
	};

};

#endif




