/************************************************************
 * Infinite Universe Engine:				                *
 *                                                          *
 * Interactive simulation of a solar system and             *
 * real-time rendering of planetary bodies with OpenGL      *
 * copyright  2002 Jochen Winzen                           *
 * winzen@online.de                                         *
 * -------------------------------------------------------- *
 * File:	Texture.h                                       *
 *                                                          *
 * Content: Header file of my own texture class with        *
 *          special functions like mipmap-blending,         *
 *          transparency... supports bmp, tga, jpeg         *
 *                                                          *
 * Author:  Jochen Winzen                                   *
 * Version: 2002/12/08                                      *
 ************************************************************
 */
#ifndef TEXTURE_H
#define TEXTURE_H

#include "Noise.h"
#include <qgl>
#include <stdio>

class TEXTURE{
 public:
	 typedef enum TEX_MODE { T_NORMAL=1, T_TERRAIN=2, T_NOMIPMAP=3, T_DETAIL=4, T_CUBEMAP=5, T_CHAOS=6 };
	 typedef enum TEX_MIPMAP { MIPMAP_NORMAL=1, MIPMAP_WHITE=2, MIPMAP_DECAL=3, MIPMAP_CHAOS=4 };

	TEXTURE();
	~TEXTURE();

  static void build2DMipmaps(GLint type, int width, int height, GLubyte *imageData, TEX_MIPMAP mode = MIPMAP_NORMAL);

  void init(int w, int h, GLuint);
	void createFractal(int, int, GLuint);
	void activate(TEX_MODE);
	bool LoadImage(char *, bool transparent = false);
	bool LoadTGA(char *);
	bool LoadBMP(char *, bool);
	//bool WriteBMP(char *filename);
	bool LoadJPEG(char * filename);
	bool WriteJPEG (char * filename, int quality = 80);
		
	GLubyte	* imageData;								// Image Data (Up To 32 Bits)
	GLuint	bpp;										// Image Color Depth In Bits Per Pixel
	GLuint	width;										// Image Width
	GLuint	height;										// Image Height
	GLuint	texID;										// Texture ID Used To Select A Texture
	GLuint	type;										// Image Type (GL_RGB, GL_RGBA)

	bool keepRawData;
	
 private:
	 bool LoadUncompressedTGA(char *, FILE *);	// Load an Uncompressed file
	 bool LoadCompressedTGA(char *, FILE *);		// Load a Compressed file
};

typedef struct
{
	GLubyte Header[12];									// TGA File Header
} TGAHeader;


typedef struct
{
	GLubyte		header[6];								// First 6 Useful Bytes From The Header
	GLuint		bytesPerPixel;							// Holds Number Of Bytes Per Pixel Used In The TGA File
	GLuint		imageSize;								// Used To Store The Image Size When Setting Aside Ram
	GLuint		temp;									// Temporary Variable
	GLuint		type;	
	GLuint		Height;									//Height of Image
	GLuint		Width;									//Width ofImage
	GLuint		Bpp;									// Bits Per Pixel
} TGA;

#endif 


