/************************************************************
 * Infinite Universe Engine:                                *
 *                                                          *
 * Interactive simulation of a solar system and             *
 * real-time rendering of planetary bodies with OpenGL      *
 * copyright  2002 Jochen Winzen                           *
 * winzen@online.de                                         *
 * -------------------------------------------------------- *
 * File:    main.cpp                                        *
 *                                                          *
 * Content: Implementation of the main window handling.     *
 *          Thanks to Jeff Molofee for his comprehensive    *
 *          tutorials at nehe.gamedev.net!                  *
 *          Thanks to Sean O'Neil for his beautiful         *
 *          math classes in "The Universe Development Kit"  *
 *          Thanks to Trolltech for QT!                     *
 *                                                          *
 * Author:  Jochen Winzen                                   *
 * Version: 2002/12/08                                      *
 ************************************************************
 */

#include <qapplication.h>
#include "GUI/JWInfiniteUniverseEngine.h"
#include <qtimer.h>
#include <qlayout.h>
#include <qgl.h>
//#include "glbox.h"
#include "JWGameEngine.h"

int main( int argc, char** argv )
{
	QApplication app( argc, argv );

	// Here we will use a subclass of a designed GUI window called "InfiniteUniverseEngine.ui"
	// this window will contain a menubar with popup menues, configuration and file dialogs etc!
	// It also contains a frame "glframe" that holds the OpenGL rendering device!
	JWInfiniteUniverseEngine window;
	window.setCaption(QString("Infinite Universe Engine (c) 2003 Jochen Winzen"));
	jwengine::mainwindow = &window;

	// Here we create a OpenGL widget (JWGameEngine) that renders our universe
	JWGameEngine* jwgame = new JWGameEngine((QWidget *)window.glframe, "jwgame");
	//GLBox* jwgame = new GLBox((QWidget *)window.glframe, "jwgame");
    // Put the GL widget inside the frame
    QHBoxLayout* flayout = new QHBoxLayout( (QWidget *)window.glframe, 0, 0, "flayout");
    flayout->addWidget( jwgame, 1 );

	QTimer *animationTimer = new QTimer( jwgame, "movement handler" );
    QObject::connect( animationTimer, SIGNAL(timeout()), jwgame, SLOT(animate()) );
	animationTimer->start(10, FALSE);

	app.setMainWidget(&window);

	window.show();

	return app.exec();
}




