/************************************************************
 * Infinite Universe Engine:				                *
 *                                                          *
 * Interactive simulation of a solar system and             *
 * real-time rendering of planetary bodies with OpenGL      *
 * copyright  2002 Jochen Winzen                           *
 * winzen@online.de                                         *
 * -------------------------------------------------------- *
 * File:	QTXML.cpp                                       *
 *                                                          *
 * Content: Implementation of XML wrapper class to QT DOM   *
 *                                                          *
 * Author:  Jochen Winzen                                   *
 * Version: 2002/12/08                                      *
 ************************************************************
 */
#include "qtxml.h"
#include <qapplication.h>
#include <qfile.h>

bool QTXML::readFile(const char *file)
{
	QFile f( file );

	if ( !f.open( IO_ReadOnly ) )
	{
		qWarning("Could not open file %s", file);
		return false;	
	}
	if ( !doc.setContent( &f ) )
	{
		qWarning("Could not parse file %s", file);
		return false;
	}

	f.close();

	docElem = doc.documentElement();
	return true;
}

QDomNode QTXML::readUniqueNode(std::string path)
{
	int pos_beg = 0;
	int pos_end = path.find(":");
	std::string elemToFind = path.substr(pos_beg, pos_end);
	
	// now search the unique element defined by path from top level of document
	QDomNode node = docElem;
	
	while( !node.isNull())
	{
		const char* chNodeName = node.nodeName();
		
		qWarning("searching[%s] at node[%s]", elemToFind.c_str(), chNodeName);

		if( strcmp(chNodeName, elemToFind.c_str()) == 0 )
		{
			if( pos_end == -1 ) // We're done!
				break;
			node = node.firstChild();
			pos_beg = pos_end + 1;
			pos_end = path.find(":", pos_beg);
			elemToFind = path.substr(pos_beg, pos_end - pos_beg);
		}			
		else
			node = node.nextSibling();
	}
	
	return node;
}


void QTXML::getColor(QDomElement n, double &red, double &green, double &blue)
{
	// get the red value
	if (n.hasAttribute("RED"))
		red = (n.attribute("RED")).toDouble();
	// get the green value
	if (n.hasAttribute("GREEN"))
		green = (n.attribute("GREEN")).toDouble();
	// get the blue value
	if (n.hasAttribute("BLUE"))
		blue = (n.attribute("BLUE")).toDouble();
}



